"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var image_source_1 = require("@nativescript/core/image-source/image-source");
var http_common_1 = require("./http.common");
var types = require("@nativescript/core/utils/types");
var domainDebugger = require("tns-core-modules/debugger");
var fs = require("tns-core-modules/file-system");
var http_common_2 = require("./http.common");
exports.HTTPFormData = http_common_2.HTTPFormData;
exports.HTTPFormDataEntry = http_common_2.HTTPFormDataEntry;
exports.ImageParseMethod = http_common_2.ImageParseMethod;
var HttpResponseEncoding;
(function (HttpResponseEncoding) {
    HttpResponseEncoding[HttpResponseEncoding["UTF8"] = 0] = "UTF8";
    HttpResponseEncoding[HttpResponseEncoding["GBK"] = 1] = "GBK";
})(HttpResponseEncoding = exports.HttpResponseEncoding || (exports.HttpResponseEncoding = {}));
var currentDevice = UIDevice.currentDevice;
var device = currentDevice.userInterfaceIdiom === 0 ? "Phone" : "Pad";
var osVersion = currentDevice.systemVersion;
var customUserAgent;
var GET = "GET";
exports.USER_AGENT_HEADER = "User-Agent";
exports.USER_AGENT = "Mozilla/5.0 (i" + device + "; CPU OS " + osVersion.replace(".", "_") + " like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/" + osVersion + " Mobile/10A5355d Safari/8536.25";
var sessionConfig = NSURLSessionConfiguration.defaultSessionConfiguration;
var queue = NSOperationQueue.mainQueue;
var certificatePinningInstance = null;
var certificatePinningConfig = null;
var certificatePinningDomainList = null;
function parseJSON(source) {
    var src = source.trim();
    if (src.lastIndexOf(")") === src.length - 1) {
        return JSON.parse(src.substring(src.indexOf("(") + 1, src.lastIndexOf(")")));
    }
    return JSON.parse(src);
}
var NSURLSessionTaskDelegateImpl = (function (_super) {
    __extends(NSURLSessionTaskDelegateImpl, _super);
    function NSURLSessionTaskDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NSURLSessionTaskDelegateImpl.prototype.URLSessionTaskDidReceiveChallengeCompletionHandler = function (session, task, challenge, completionHandler) {
        if (certificatePinningInstance == null) {
            completionHandler(1, null);
            return;
        }
        var pinningValidator = certificatePinningInstance.pinningValidator;
        if (!pinningValidator.handleChallengeCompletionHandler(challenge, completionHandler)) {
            completionHandler(1, null);
        }
    };
    NSURLSessionTaskDelegateImpl.ObjCProtocols = [NSURLSessionTaskDelegate];
    return NSURLSessionTaskDelegateImpl;
}(NSObject));
var sessionTaskDelegateInstance = NSURLSessionTaskDelegateImpl.new();
var NoRedirectNSURLSessionTaskDelegateImpl = (function (_super) {
    __extends(NoRedirectNSURLSessionTaskDelegateImpl, _super);
    function NoRedirectNSURLSessionTaskDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NoRedirectNSURLSessionTaskDelegateImpl.prototype.URLSessionTaskDidReceiveChallengeCompletionHandler = function (session, task, challenge, completionHandler) {
        if (certificatePinningInstance == null) {
            completionHandler(1, null);
            return;
        }
        var pinningValidator = certificatePinningInstance.pinningValidator;
        if (!pinningValidator.handleChallengeCompletionHandler(challenge, completionHandler)) {
            completionHandler(1, null);
        }
    };
    NoRedirectNSURLSessionTaskDelegateImpl.prototype.URLSessionTaskWillPerformHTTPRedirectionNewRequestCompletionHandler = function (session, task, response, request, completionHandler) {
        completionHandler(null);
    };
    NoRedirectNSURLSessionTaskDelegateImpl.ObjCProtocols = [NSURLSessionTaskDelegate];
    return NoRedirectNSURLSessionTaskDelegateImpl;
}(NSObject));
var noRedirectSessionTaskDelegateInstance = NoRedirectNSURLSessionTaskDelegateImpl.new();
var defaultSession;
function ensureDefaultSession() {
    if (!defaultSession) {
        defaultSession = NSURLSession.sessionWithConfigurationDelegateDelegateQueue(sessionConfig, sessionTaskDelegateInstance, queue);
    }
}
var sessionNotFollowingRedirects;
function ensureSessionNotFollowingRedirects() {
    if (!sessionNotFollowingRedirects) {
        sessionNotFollowingRedirects = NSURLSession.sessionWithConfigurationDelegateDelegateQueue(sessionConfig, noRedirectSessionTaskDelegateInstance, queue);
    }
}
function request(options) {
    if (options === undefined || options === null) {
        return Promise.reject("No options given");
    }
    if (options.url === "https://nativescript-http-integration-check.local") {
        return http_common_1.completeSelfCheck(options);
    }
    return new Promise(function (resolve, reject) {
        if (!options.url) {
            reject(new Error("Request url was empty."));
            return;
        }
        try {
            var network = domainDebugger.getNetwork();
            var debugRequest_1 = network && network.create();
            var urlRequest = NSMutableURLRequest.requestWithURL(NSURL.URLWithString(options.url));
            urlRequest.HTTPMethod = types.isDefined(options.method) ? options.method : GET;
            var contentType = void 0;
            var userAgent = void 0;
            if (options.headers) {
                for (var key in options.headers) {
                    if (key.toLowerCase() === "content-type") {
                        contentType = options.headers[key];
                    }
                    if (key.toLowerCase() === "user-agent") {
                        userAgent = options.headers[key];
                    }
                }
            }
            if (!userAgent) {
                if (!options.headers) {
                    options.headers = {};
                }
                options.headers[exports.USER_AGENT_HEADER] = customUserAgent ? customUserAgent : exports.USER_AGENT;
            }
            if (options.headers) {
                for (var header in options.headers) {
                    urlRequest.setValueForHTTPHeaderField(options.headers[header] + "", header);
                }
            }
            if (types.isString(options.content) || options.content instanceof FormData) {
                urlRequest.HTTPBody = NSString.stringWithString(options.content.toString()).dataUsingEncoding(4);
            }
            else if (options.content instanceof ArrayBuffer) {
                var buffer = options.content;
                urlRequest.HTTPBody = NSData.dataWithData(buffer);
            }
            else {
                var matched = false;
                if (!matched && options.content instanceof Blob) {
                    if (!contentType && options.content.type) {
                        urlRequest.setValueForHTTPHeaderField(options.content.type, "Content-Type");
                    }
                    var buffer = new Uint8Array(Blob.InternalAccessor.getBuffer(options.content).buffer.slice(0));
                    urlRequest.HTTPBody = NSData.dataWithData(buffer);
                    matched = true;
                }
                if (!matched && options.content instanceof http_common_1.HTTPFormData) {
                    matched = true;
                    var multipartFormData_1 = OMGMultipartFormData.new();
                    var contentValues = options.content;
                    contentValues.forEach((function (value, key) {
                        if (typeof value === "string") {
                            multipartFormData_1.addTextParameterName(value, key);
                        }
                        else if (value instanceof ArrayBuffer) {
                            var buffer = new Uint8Array(value);
                            multipartFormData_1.addFileParameterNameFilenameContentType(NSData.dataWithData(buffer), key, "", "application/octet-stream");
                        }
                        else if (value instanceof Blob) {
                            var formDataPartMediaType = "application/octet-stream";
                            if (value.type) {
                                formDataPartMediaType = value.type;
                            }
                            var filename = "";
                            if (value instanceof File) {
                                filename = value.name;
                            }
                            var buffer = new Uint8Array(Blob.InternalAccessor.getBuffer(value).buffer.slice(0));
                            multipartFormData_1.addFileParameterNameFilenameContentType(NSData.dataWithData(buffer), key, filename, formDataPartMediaType);
                        }
                        else if (value instanceof http_common_1.HTTPFormDataEntry) {
                            var formDataPartMediaType = "application/octet-stream";
                            if (value.type) {
                                formDataPartMediaType = value.type;
                            }
                            var filename = value.name || "";
                            if (value.data instanceof ArrayBuffer) {
                                var buffer = new Uint8Array(value.data);
                                multipartFormData_1.addFileParameterNameFilenameContentType(NSData.dataWithData(buffer), key, filename, formDataPartMediaType);
                            }
                            else if (value.data instanceof Blob) {
                                var buffer = new Uint8Array(Blob.InternalAccessor.getBuffer(value.data).buffer.slice(0));
                                multipartFormData_1.addFileParameterNameFilenameContentType(NSData.dataWithData(buffer), key, filename, formDataPartMediaType);
                            }
                            else {
                                multipartFormData_1.addFileParameterNameFilenameContentType(value.data, key, filename, formDataPartMediaType);
                            }
                        }
                        else {
                            multipartFormData_1.addFileParameterNameFilenameContentType(value, key, "", "application/octet-stream");
                        }
                    }));
                    var multipartRequest = OMGHTTPURLRQ.POSTError(options.url, multipartFormData_1);
                    var multiPartContentType = multipartRequest.allHTTPHeaderFields.objectForKey("Content-Type");
                    urlRequest.setValueForHTTPHeaderField(multiPartContentType, "Content-Type");
                    urlRequest.HTTPBody = NSData.dataWithData(multipartRequest.HTTPBody);
                }
                if (!matched && options.content) {
                    urlRequest.HTTPBody = NSData.dataWithData(options.content);
                }
            }
            if (types.isNumber(options.timeout)) {
                urlRequest.timeoutInterval = options.timeout / 1000;
            }
            var session = void 0;
            if (types.isBoolean(options.dontFollowRedirects) && options.dontFollowRedirects) {
                ensureSessionNotFollowingRedirects();
                session = sessionNotFollowingRedirects;
            }
            else {
                ensureDefaultSession();
                session = defaultSession;
            }
            var dataTask = session.dataTaskWithRequestCompletionHandler(urlRequest, function (data, response, error) {
                if (error) {
                    reject(new Error(error.localizedDescription));
                }
                else {
                    var headers_1 = {};
                    if (response && response.allHeaderFields) {
                        var headerFields = response.allHeaderFields;
                        headerFields.enumerateKeysAndObjectsUsingBlock(function (key, value, stop) {
                            addHeader(headers_1, key, value);
                        });
                    }
                    if (debugRequest_1) {
                        debugRequest_1.mimeType = response.MIMEType;
                        debugRequest_1.data = data;
                        var debugResponse = {
                            url: options.url,
                            status: response.statusCode,
                            statusText: NSHTTPURLResponse.localizedStringForStatusCode(response.statusCode),
                            headers: headers_1,
                            mimeType: response.MIMEType,
                            fromDiskCache: false
                        };
                        debugRequest_1.responseReceived(debugResponse);
                        debugRequest_1.loadingFinished();
                    }
                    resolve({
                        content: {
                            raw: data,
                            toArrayBuffer: function () { return interop.bufferFromData(data); },
                            toString: function (encoding) {
                                var str = NSDataToString(data, encoding);
                                if (typeof str === "string") {
                                    return str;
                                }
                                else {
                                    throw new Error("Response content may not be converted to string");
                                }
                            },
                            toJSON: function (encoding) { return parseJSON(NSDataToString(data, encoding)); },
                            toImage: function () {
                                return new Promise(function (resolve, reject) {
                                    UIImage.tns_decodeImageWithDataCompletion(data, function (image) {
                                        if (image) {
                                            resolve(new image_source_1.ImageSource(image));
                                        }
                                        else {
                                            reject(new Error("Response content may not be converted to an Image"));
                                        }
                                    });
                                });
                            },
                            toFile: function (destinationFilePath) {
                                if (!destinationFilePath) {
                                    destinationFilePath = http_common_1.getFilenameFromUrl(options.url);
                                }
                                if (data instanceof NSData) {
                                    var file = fs.File.fromPath(destinationFilePath);
                                    data.writeToFileAtomically(destinationFilePath, true);
                                    return file;
                                }
                                else {
                                    reject(new Error("Cannot save file with path: " + destinationFilePath + "."));
                                }
                                return null;
                            }
                        },
                        statusCode: response.statusCode,
                        headers: headers_1
                    });
                }
            });
            if (options.url && debugRequest_1) {
                var request_1 = {
                    url: options.url,
                    method: "GET",
                    headers: options.headers
                };
                debugRequest_1.requestWillBeSent(request_1);
            }
            dataTask.resume();
        }
        catch (ex) {
            reject(ex);
        }
    });
}
exports.request = request;
function NSDataToString(data, encoding) {
    var code = NSUTF8StringEncoding;
    if (encoding === HttpResponseEncoding.GBK) {
        code = 1586;
    }
    var encodedString = NSString.alloc().initWithDataEncoding(data, code);
    if (!encodedString) {
        code = NSISOLatin1StringEncoding;
        encodedString = NSString.alloc().initWithDataEncoding(data, code);
    }
    return encodedString.toString();
}
function getString(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var str = r.content.toString();
                resolve(str);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getString = getString;
function getJSON(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var json = r.content.toJSON();
                resolve(json);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getJSON = getJSON;
function getImage(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                resolve(r.content.toImage());
            }
            catch (err) {
                reject(err);
            }
        }, function (err) {
            reject(err);
        });
    });
}
exports.getImage = getImage;
function getFile(arg, destinationFilePath) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var file = r.content.toFile(destinationFilePath);
                resolve(file);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getFile = getFile;
function getBinary(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var arrayBuffer = r.content.toArrayBuffer();
                resolve(arrayBuffer);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getBinary = getBinary;
function addHeader(headers, key, value) {
    if (!headers[key]) {
        headers[key] = value;
    }
    else if (Array.isArray(headers[key])) {
        headers[key].push(value);
    }
    else {
        var values = [headers[key]];
        values.push(value);
        headers[key] = values;
    }
}
exports.addHeader = addHeader;
function setImageParseMethod(imageParseMethod) {
}
exports.setImageParseMethod = setImageParseMethod;
function setConcurrencyLimits(maxRequests, maxRequestsPerHost) {
    sessionConfig.HTTPMaximumConnectionsPerHost = maxRequestsPerHost;
}
exports.setConcurrencyLimits = setConcurrencyLimits;
function clearCookies() {
    if (sessionConfig.HTTPCookieStorage && sessionConfig.HTTPCookieStorage.cookies) {
        var cookieCount = sessionConfig.HTTPCookieStorage.cookies.count;
        if (cookieCount <= 0) {
            return;
        }
        for (var i = 0; i < cookieCount; i++) {
            var cookie = sessionConfig.HTTPCookieStorage.cookies.objectAtIndex(0);
            if (cookie) {
                sessionConfig.HTTPCookieStorage.deleteCookie(cookie);
            }
        }
    }
}
exports.clearCookies = clearCookies;
function setUserAgent(userAgent) {
    customUserAgent = userAgent;
}
exports.setUserAgent = setUserAgent;
function certificatePinningAdd(pattern, hashes) {
    if (sessionConfig.URLCache) {
        sessionConfig.URLCache.removeAllCachedResponses();
    }
    if (certificatePinningConfig == null) {
        certificatePinningConfig = NSMutableDictionary.new().init();
        certificatePinningConfig.setValueForKey(false, kTSKSwizzleNetworkDelegates);
    }
    if (certificatePinningDomainList == null) {
        certificatePinningDomainList = NSMutableDictionary.new().init();
    }
    var domainSetting = NSMutableDictionary.new().init();
    domainSetting.setValueForKey(true, kTSKDisableDefaultReportUri);
    domainSetting.setValueForKey(hashes, kTSKPublicKeyHashes);
    var allowSubdomains = false;
    var domain = pattern;
    if (domain.indexOf("**.") === 0) {
        domain = domain.slice("**.".length);
        allowSubdomains = true;
    }
    if (domain.indexOf("*.") === 0) {
        domain = domain.slice("*.".length);
        allowSubdomains = true;
    }
    domainSetting.setValueForKey(allowSubdomains, kTSKIncludeSubdomains);
    certificatePinningDomainList.setValueForKey(domainSetting, domain);
    certificatePinningConfig.setValueForKey(certificatePinningDomainList, kTSKPinnedDomains);
    certificatePinningInstance = TrustKit.alloc().initWithConfiguration(certificatePinningConfig);
}
exports.certificatePinningAdd = certificatePinningAdd;
function certificatePinningClear() {
    if (sessionConfig.URLCache) {
        sessionConfig.URLCache.removeAllCachedResponses();
    }
    certificatePinningConfig = null;
    certificatePinningInstance = null;
    certificatePinningDomainList = null;
}
exports.certificatePinningClear = certificatePinningClear;
function getCurrentUserAgent() {
    return customUserAgent;
}
exports.getCurrentUserAgent = getCurrentUserAgent;
function getCurrentCertificatePinningInstance() {
    return certificatePinningInstance;
}
exports.getCurrentCertificatePinningInstance = getCurrentCertificatePinningInstance;
//# sourceMappingURL=http.ios.js.map